@@ -172,7 +172,7 @@ void AutoTypePlatformMac::sendChar(const QChar& ch, bool isKeyDown)
 // Send key code to active window
 // see: Quartz Event Services
 //
-void AutoTypePlatformMac::sendKey(Qt::Key key, bool isKeyDown, Qt::KeyboardModifiers modifiers = 0)
+void AutoTypePlatformMac::sendKey(Qt::Key key, bool isKeyDown, Qt::KeyboardModifiers modifiers)
 {
     uint16 keyCode = macUtils()->qtToNativeKeyCode(key);
     if (keyCode == INVALID_KEYCODE) {
@@ -238,38 +238,22 @@ AutoTypeAction::Result AutoTypeExecutorMac::execBegin(const AutoTypeBegin* actio
 
 AutoTypeAction::Result AutoTypeExecutorMac::execType(const AutoTypeKey* action)
 {
-    if (action->modifiers & Qt::ShiftModifier) {
-        m_platform->sendKey(Qt::Key_Shift, true);
-    }
-    if (action->modifiers & Qt::ControlModifier) {
-        m_platform->sendKey(Qt::Key_Control, true);
-    }
-    if (action->modifiers & Qt::AltModifier) {
-        m_platform->sendKey(Qt::Key_Alt, true);
-    }
-    if (action->modifiers & Qt::MetaModifier) {
-        m_platform->sendKey(Qt::Key_Meta, true);
-    }
+
 
     if (action->key != Qt::Key_unknown) {
-        m_platform->sendKey(action->key, true);
-        m_platform->sendKey(action->key, false);
+        m_platform->sendKey(action->key, true, action->modifiers);
+        m_platform->sendKey(action->key, false, action->modifiers);
     } else {
-        m_platform->sendChar(action->character, true);
-        m_platform->sendChar(action->character, false);
-    }
-
-    if (action->modifiers & Qt::ShiftModifier) {
-        m_platform->sendKey(Qt::Key_Shift, false);
-    }
-    if (action->modifiers & Qt::ControlModifier) {
-        m_platform->sendKey(Qt::Key_Control, false);
-    }
-    if (action->modifiers & Qt::AltModifier) {
-        m_platform->sendKey(Qt::Key_Alt, false);
-    }
-    if (action->modifiers & Qt::MetaModifier) {
-        m_platform->sendKey(Qt::Key_Meta, false);
+        if (action->modifiers != Qt::NoModifier) {
+            // If we have modifiers set than we intend to send a key sequence
+            // convert to uppercase to align with Qt Key mappings
+            int ch = action->character.toUpper().toLatin1();
+            m_platform->sendKey(static_cast<Qt::Key>(ch), true, action->modifiers);
+            m_platform->sendKey(static_cast<Qt::Key>(ch), false, action->modifiers);
+        } else {
+            m_platform->sendChar(action->character, true);
+            m_platform->sendChar(action->character, false);
+        }
     }
 
     Tools::sleep(execDelayMs);
