@@ -17,8 +17,12 @@
 
 #include "Utils.h"
 
+#include "core/Database.h"
+#include "core/Entry.h"
+#include "core/EntryAttributes.h"
+#include "keys/FileKey.h"
 #ifdef WITH_XC_YUBIKEY
-#include "keys/YkChallengeResponseKeyCLI.h"
+#include "keys/ChallengeResponseKey.h"
 #endif
 
 #ifdef Q_OS_WIN
@@ -30,7 +34,6 @@
 
 #include <QFileInfo>
 #include <QProcess>
-#include <QScopedPointer>
 
 namespace Utils
 {
@@ -91,7 +94,7 @@ namespace Utils
     }
 
     QSharedPointer<Database> unlockDatabase(const QString& databaseFilename,
-                                            const bool isPasswordProtected,
+                                            bool isPasswordProtected,
                                             const QString& keyFilename,
                                             const QString& yubiKeySlot,
                                             bool quiet)
@@ -165,9 +168,14 @@ namespace Utils
                 }
             }
 
-            auto key = QSharedPointer<YkChallengeResponseKeyCLI>(new YkChallengeResponseKeyCLI(
-                {serial, slot}, QObject::tr("Please touch the button on your YubiKey to continue…"), err));
+            QObject::connect(YubiKey::instance(), &YubiKey::userInteractionRequest, [&] {
+                err << QObject::tr("Please present or touch your YubiKey to continue.") << "\n\n" << flush;
+            });
+
+            auto key = QSharedPointer<ChallengeResponseKey>(new ChallengeResponseKey({serial, slot}));
             compositeKey->addChallengeResponseKey(key);
+
+            YubiKey::instance()->findValidKeys();
         }
 #else
         Q_UNUSED(yubiKeySlot);
@@ -175,7 +183,7 @@ namespace Utils
 
         auto db = QSharedPointer<Database>::create();
         QString error;
-        if (db->open(databaseFilename, compositeKey, &error, false)) {
+        if (db->open(databaseFilename, compositeKey, &error)) {
             return db;
         } else {
             err << error << endl;
@@ -191,6 +199,12 @@ namespace Utils
      */
     QString getPassword(bool quiet)
     {
+#ifdef __AFL_COMPILER
+        // Fuzz test build takes password from environment variable to
+        // allow non-interactive operation
+        const auto env = getenv("KEYPASSXC_AFL_PASSWORD");
+        return env ? env : "";
+#else
         auto& in = STDIN;
         auto& out = quiet ? DEVNULL : STDERR;
 
@@ -200,6 +214,7 @@ namespace Utils
         out << endl;
 
         return line;
+#endif // __AFL_COMPILER
     }
 
     /**
@@ -255,7 +270,7 @@ namespace Utils
 
 #ifdef Q_OS_UNIX
         if (QProcessEnvironment::systemEnvironment().contains("WAYLAND_DISPLAY")) {
-            clipPrograms << qMakePair(QStringLiteral("wl-copy"), QStringLiteral(""));
+            clipPrograms << qMakePair(QStringLiteral("wl-copy"), QStringLiteral("-t text/plain"));
         } else {
             clipPrograms << qMakePair(QStringLiteral("xclip"), QStringLiteral("-selection clipboard -i"));
         }
@@ -277,7 +292,7 @@ namespace Utils
 
         QStringList failedProgramNames;
 
-        for (auto prog : clipPrograms) {
+        for (const auto& prog : clipPrograms) {
             QScopedPointer<QProcess> clipProcess(new QProcess(nullptr));
 
             // Skip empty parts, otherwise the program may clip the empty string
@@ -291,8 +306,15 @@ namespace Utils
                 continue;
             }
 
-            if (clipProcess->write(text.toLatin1()) == -1) {
-                qDebug("Unable to write to process : %s", qPrintable(clipProcess->errorString()));
+#ifdef Q_OS_WIN
+            // Windows clip command only understands Unicode written as UTF-16
+            auto data = QByteArray::fromRawData(reinterpret_cast<const char*>(text.utf16()), text.size() * 2);
+            if (clipProcess->write(data) == -1) {
+#else
+            // Other platforms understand UTF-8
+            if (clipProcess->write(text.toUtf8()) == -1) {
+#endif
+                qWarning("Unable to write to process : %s", qPrintable(clipProcess->errorString()));
             }
             clipProcess->waitForBytesWritten();
             clipProcess->closeWriteChannel();
@@ -347,6 +369,17 @@ namespace Utils
         return result;
     }
 
+    QString getTopLevelField(const Entry* entry, const QString& fieldName)
+    {
+        if (fieldName == UuidFieldName) {
+            return entry->uuid().toString();
+        }
+        if (fieldName == TagsFieldName) {
+            return entry->tags();
+        }
+        return QString("");
+    }
+
     QStringList findAttributes(const EntryAttributes& attributes, const QString& name)
     {
         QStringList result;
@@ -364,4 +397,37 @@ namespace Utils
         return result;
     }
 
+    /**
+     * Load a key file from disk. When the path specified does not exist a
+     * new file will be generated. No folders will be generated so the parent
+     * folder of the specified file needs to exist
+     *
+     * If the key file cannot be loaded or created the function will fail.
+     *
+     * @param path Path to the key file to be loaded
+     * @param fileKey Resulting fileKey
+     * @return true if the key file was loaded succesfully
+     */
+    bool loadFileKey(const QString& path, QSharedPointer<FileKey>& fileKey)
+    {
+        auto& err = Utils::STDERR;
+        QString error;
+        fileKey = QSharedPointer<FileKey>(new FileKey());
+
+        if (!QFileInfo::exists(path)) {
+            fileKey->create(path, &error);
+
+            if (!error.isEmpty()) {
+                err << QObject::tr("Creating KeyFile %1 failed: %2").arg(path, error) << endl;
+                return false;
+            }
+        }
+
+        if (!fileKey->load(path, &error)) {
+            err << QObject::tr("Loading KeyFile %1 failed: %2").arg(path, error) << endl;
+            return false;
+        }
+
+        return true;
+    }
 } // namespace Utils
